﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;
using System.Runtime.CompilerServices;

namespace wms.dto.request.hj.dto
{
    /// <summary>
    /// 仓库查询请求参数
    /// </summary>
    public class QuertRequest : BaseRequest
    {
        public string? Code { get; set; }
        public string? Name { get; set; }
        public string? WareState { get; set; }
        public string? WareType { get; set; }
        public string? FactoryNo { get; set; }

    }
    /// <summary>
    /// 库区查询请求参数
    /// </summary>
    public class WareZoneRequest : BaseRequest
    {
        /// <summary>
        /// 库区编码
        /// </summary>
        public string Code { get; set; }
        /// <summary>
        /// 库区名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 仓库id
        /// </summary>
        public long WareId { get; set; }
        /// <summary>
        ///  库区状态
        /// </summary>
        public string WZoneState { get; set; }
    }

    /// <summary>
    /// 货位查询请求参数
    /// </summary>
    public class WareLocationRequest : BaseRequest
    {
        public string Code { get; set; }
        public string Name { get; set; }
        public int LocationType { get; set; }
        public int Tunnel { get; set; }
        public string WareCode { get; set; }
    }
    /// <summary>
    /// 单位查询请求参数
    /// </summary>
    public class BaseUnitRequest : BaseRequest
    {
        public string Code { get; set; }
        public string Name { get; set; }
        public string PrimaryUnit { get; set; }
        public string DeputyUnit { get; set; }
    }
    /// <summary>
    /// 物料查询请求参数
    /// </summary>
    public class BaseMaterRequest : BaseRequest
    {
        public string Code { get; set; }
        public string Name { get; set; }
        public bool? IsHold { get; set; }
        public decimal HoldDuration { get; set; }
        public string Strength { get; set; }
        public string Plating { get; set; }
        public decimal PlatingWeight { get; set; }
    }
    /// <summary>
    /// 条码查询请求参数
    /// </summary>
    public class BaseBarCodeRequest : BaseRequest
    {
        public string MatCode { get; set; }
        public string HWBarCode { get; set; }
        public string RFIDBarCode { get; set; }
        public string InDocsNo { get; set; }
    }
    /// <summary>
    /// 报表参数请求
    /// </summary>
    public class ReportRequest : BaseRequest
    {
        /// <summary>
        /// 容器id
        /// </summary>
        public string? RFID { get; set; }
        /// <summary>
        /// 工字轮型号
        /// </summary>
        public string? HWSpec { get; set; }
        /// <summary>
        /// 物料等级
        /// </summary>
        public string? Grade { get; set; }
        /// <summary>
        /// 执行单据号
        /// </summary>
        public string? ExecDocsNo { get; set; }
        /// <summary>
        /// 摆放位置编号
        /// </summary>
        public string? PutLocNo { get; set; }
        /// <summary>
        /// 任务号
        /// </summary>
        public int TaskCode { get; set; }
        /// <summary>
        /// 巷道
        /// </summary>
        public string? Tunnel { get; set; }
        /// <summary>
        /// 起始地址
        /// </summary>
        public string? StartPoint { get; set; }
        /// <summary>
        /// 目标地址
        /// </summary>
        public string? EndPoint { get; set; }
        /// <summary>
        /// 货位号
        /// </summary>
        public string? Location { get; set; }
        /// <summary>
        /// 容器条码
        /// </summary>
        public string? ContGrpBarCode { get; set; }
        /// <summary>
        /// 材料号
        /// </summary>
        public string? InvBarCode { get; set; }
        /// <summary>
        /// 物料编码
        /// </summary>
        public string? MatCode { get; set; }
        /// <summary>
        /// 时效
        /// </summary>
        public decimal? HoldDuration { get; set; }
    }
    /// <summary>
    /// 任务参数请求
    /// </summary>
    public class TaskRequest:BaseRequest
    {
        /// <summary>
        /// 任务号
        /// </summary>
        public int TaskCode { get; set; }
        /// <summary>
        /// 任务类型
        /// </summary>
        public int TaskType { get; set; }
        /// <summary>
        /// 任务状态
        /// </summary>
        public int TaskState { get; set; }
        /// <summary>
        /// 仓库
        /// </summary>
        public string? WareCode { get; set; }
        /// <summary>
        /// 起始地址
        /// </summary>
        public string? StartPoint { get; set; }
        /// <summary>
        /// 目标地址
        /// </summary>
        public string? EndPoint { get; set; }
        /// <summary>
        /// 物料编码
        /// </summary>
        public string? MatCode { get; set; }
        /// <summary>
        /// 容器条码
        /// </summary>
        public string? ContGrpBarCode { get; set; }
        /// <summary>
        /// 巷道
        /// </summary>
        public int Tunnel { get; set; }
        /// <summary>
        /// 任务业务类型
        /// </summary>
        public string? BusType { get; set; }
        /// <summary>
        /// 单据号
        /// </summary>
        public string? DocCode { get; set; }
    }
    /// <summary>
    /// 单据参数请求
    /// </summary>
    public class DocumentsRequest : BaseRequest
    {
        /// <summary>
        /// 单号
        /// </summary>
        public string? DocsNo { get; set; }
        /// <summary>
        /// 请求凭证号
        /// </summary>
        public string? ReqNo { get; set; }
        /// <summary>
        /// 单据类型
        /// </summary>
        public string? TypeCode { get; set; }
        /// <summary>
        /// 单据状态
        /// </summary>
        public string? StateCode { get; set; }
        public string? Rfid { get; set; }
        public string? location { get; set; }
    }
    /// <summary>
    /// 库存请求参数
    /// </summary>
    public class StockRequest : BaseRequest
    {
        /// <summary>
        /// 容器id
        /// </summary>
        public string? RFID { get; set; }
        /// <summary>
        /// 工字轮型号
        /// </summary>
        public string? HWSpec { get; set; }
        /// <summary>
        /// 物料等级
        /// </summary>
        public string? Grade { get; set; }
        /// <summary>
        /// 巷道
        /// </summary>
        public string? Tunnel { get; set; }
        /// <summary>
        /// 货位号
        /// </summary>
        public string? Location { get; set; }
        /// <summary>
        /// 容器条码
        /// </summary>
        public string? ContGrpBarCode { get; set; }
        /// <summary>
        /// 材料号
        /// </summary>
        public string? InvBarCode { get; set; }
        /// <summary>
        /// 物料编码
        /// </summary>
        public string? MatCode { get; set; }
        /// <summary>
        /// 时效
        /// </summary>
        public decimal? HoldDuration { get; set; }
        /// <summary>
        /// 组盘状态
        /// </summary>
        public int GroupSate { get; set; }
    }
    public class ManualTaskRequest
    {
        public List<string> Location { get; set; }
        //[Required(ErrorMessage = "{0} 参数不可为空")]
        //public string EndPoint { get; set; }
        public string? BusType { get; set; }
    }
    public class ExportModel
    {
        public string Title { get; set; }
        public string Code { get; set; }
    }
    /// <summary>
    /// 解绑
    /// </summary>
    public class UnboundRequest
    {
        /// <summary>
        /// 位置码
        /// </summary>
        [Required(ErrorMessage = "{0} 不可为空")]
        public string Equip { get; set; }
    }
    /// <summary>
    /// RFID
    /// </summary>
    public class PdaRfidRequest
    {
        public List<string> RFIDBarCode { get; set; }
        public string? Equip { get; set; }
        public int BusType { get; set; } = 0;
    }
    /// <summary>
    /// 报错信息查询
    /// </summary>
    public class ErrorInfoRequest
    {
        public string info { get; set; }
    }
    /// <summary>
    /// agv搬运任务请求参数
    /// </summary>
    public class AgvTaskInfoRequest
    {
        public string Rfid { get; set; }
        /// <summary>
        /// 起点
        /// </summary>
        public string StartEquip { get; set; }
        /// <summary>
        /// 终点
        /// </summary>
        public string EndEquip { get; set; }
    }
    public class CancelTaskRequest
    {
        public List<int> TaskNo { get; set; }
        public HandleTaskTypeEnum State { get; set; }
    }
    public enum HandleTaskTypeEnum
    {
        取消任务 = 106,
        完成任务 = 99,
        //重新下发入库AGV任务 = 1,
        //重新下发出库AGV任务 = 1,
    }
    public class PushDate
    {
        public int init { get; set; } = 0;
        public string? Type { get; set; }
        public int Count { get; set; }
        public List<string>? CLBarCode { get; set; }
    }
    public class BaseRequest
    {
        [RegularExpression(@"^[0-9]*$", ErrorMessage = "{0}只能是数字")]
        public int PageIndex { get; set; } = 1;
        [RegularExpression(@"^[0-9]*$", ErrorMessage = "{0}只能是数字")]
        public int PageSize { get; set; } = 10;
        public List<DateTime>? AddTime { get; set; }
        public List<DateTime>? EditTime { get; set; }
    }
}
